from .gd_api_resource import GDApiResource
from .serializer import GDSerializer
from tastypie.bundle import Bundle

class Schema(object):
    def __init__(self, kls):
        self.obj = kls()
        self._data = self.obj.build_schema()

    def __getattr__(self, name):
        return self._data.get(name, None)

    def to_dict(self):
        return self._data

    
class SchemaResource(GDApiResource):
    class Meta:
        resource_name = 'schemas'
        collection_name = 'data'
        serializer = GDSerializer(['json','html'])
        allowed_list_methods = ['GET']
        allowed_detail_methods = ['GET']

    def detail_uri_kwargs(self, bundle_or_obj):
        kwargs = {}

        if isinstance(bundle_or_obj, Bundle):
            kwargs['pk'] = bundle_or_obj.obj['id']
        else:
            kwargs['pk'] = bundle_or_obj['id']

        return kwargs
       
    def get_object_list(self, request):
        schema_list = []
        for resource in self._meta.schema_resources.values():
            new_obj = Schema(resource)
            schema_list.append(new_obj.to_dict())

        return schema_list

    def obj_get_list(self, bundle, **kwargs):
        return self.get_object_list(bundle.request)

    def resource_dehydrate(self, bundle):
        bundle.obj['links']['self'] = bundle.data['links']['self']
        bundle.data = bundle.obj
        
        return bundle

    def obj_get(self, bundle, **kwargs):
        return Schema(self._meta.schema_resources[kwargs['pk']]).to_dict()
