import json
from re import sub

from tastypie.serializers import Serializer
from tastypie.exceptions import BadRequest
from django.core.serializers import json as djangojson
from django.conf import settings


class GDSerializer(Serializer):
    def to_html(self, data, options=None):
        """
        Complying with the GD API spec.
        """
        options = options or {}

        html = '''
        <!DOCTYPE html>
        <!-- If you are reading this, there is a good chance you would prefer sending an
        "Accept: application/json" header and receiving actual JSON responses. -->
        <link rel="stylesheet" type="text/css" href="https://img1.wsimg.com/ucp/api/1.2.17/ui.css" />
        <script src="https://img1.wsimg.com/ucp/api/1.2.17/ui.js"></script>
        <script>
        var docs = "http://confluence.int.godaddy.com/";
        var schemas = "{0}";
        var data = 
            {1}
          ;
          </script>
        '''

        #return (html % self.to_json(data, options))
        schema = getattr(settings, 'SCHEMA_URL', 'http://localhost:8000/api/v1/schemas/')
        return (html.format(schema, self.to_json(data)))

    def to_json(self, data, options=None):
        """
        Camel Casing on the way out
        """
        options = options or {}
        data = self.to_simple(data,options)

        data = self._loop_through_keys_and_change_case(data)

        return djangojson.json.dumps(data, cls=djangojson.DjangoJSONEncoder,
            sort_keys=True, ensure_ascii=False)

    def from_json(self, content):
        """
        Un-Camel Case on the way in.
        """
        try:
            py_obj = json.loads(content)
            py_obj = self._loop_through_keys_and_change_case(
                py_obj, 'underscore') 
            
            return py_obj

        except ValueError:
            raise BadRequest

    def _loop_through_keys_and_change_case(self, data, case='camel'):
        """
        data should be a dictionary.. but if it isn't just give it back.
        """
        caser = case_factory(case)
        
        if isinstance(data, dict):
            for key in data.keys():
                new_key = caser.convert(key)
                data = self._key_replace(data, key, new_key)
                # from now on... all data is referenced from new_key
                if isinstance(data[new_key], dict):
                    data[new_key] = \
                        self._loop_through_keys_and_change_case(
                            data[new_key], caser.case_type)
                if isinstance(data[new_key], list):
                    new_list = []
                    for item in data[new_key]:
                        new_list.append(
                            self._loop_through_keys_and_change_case(
                                item, caser.case_type)
                        )
                    data[new_key] = new_list

        return data   
        
    def _key_replace(self, dictionary, old_key, new_key):
        if new_key !=  old_key:
            dictionary[new_key] = dictionary[old_key]
            del(dictionary[old_key])

        return dictionary

def case_factory(type):
    if type == 'camel': return CamelCaser()
    if type == 'underscore': return UnderscoreCaser()
    assert 0, "Bad case type: " + type

class CamelCaser(object):
    def __init__(self):
        self.case_type = 'camel'

    def convert(self, value):
        value = ''.join(x for x in value.title() if x != '_')
        value = value[:1].lower() + value[1:]

        return value

class UnderscoreCaser(object):
    def __init__(self):
        self.case_type = 'underscore'

    def convert(self, value):
        first_pass = sub('(.)([A-Z][a-z]+)',r'\1_\2', value)
        return sub('([a-z0-9])([A-Z])', r'\1_\2',first_pass).lower()

